import {Component, Input, OnInit, ViewChild} from '@angular/core';
import {IonInput, ModalController} from '@ionic/angular';
import {ApiService} from '../../service/api.service';
import {BehaviorSubject} from 'rxjs';
import {Router} from '@angular/router';
import {NodeTypeIcons} from '../../structures/node-types';
import {debounce} from '../../utility';
import {DatabasePageComponent} from '../editor/database/database-page.component';
import {FileBoxPageComponent} from '../nodes/file-box/file-box-page.component';

export interface SearchResult {
    title: string;
    short_title: string;
    type: string;
    id: string;
    boxId?: string;
    associated?: {
        title: string,
        type: 'page',
        id: string
    };
}

@Component({
    selector: 'noded-search-modal',
    templateUrl: './Search.component.html',
    styleUrls: ['./Search.component.scss'],
})
export class SearchComponent implements OnInit {
    @ViewChild('ionInput') ionInput: IonInput;
    @Input() query = '';
    public results: BehaviorSubject<SearchResult[]> = new BehaviorSubject<SearchResult[]>([]);
    public loading = false;

    public typeIcons = NodeTypeIcons;

    protected searchChangeDebounce = debounce(async ($event) => {
        const query = $event.detail.value;
        this.results.next(await this.search(query));
        this.loading = false;
    }, 1000);

    constructor(
        protected ionModalController: ModalController,
        protected api: ApiService,
        protected router: Router,
    ) { }

    async dismiss() {
        await this.ionModalController.dismiss();
    }

    ngOnInit() {
        setTimeout(() => {
            this.ionInput.setFocus();
        }, 400);
    }

    async onSearchChange($event) {
        this.loading = true;
        this.searchChangeDebounce($event);
    }

    async search(query): Promise<SearchResult[]> {
        return new Promise(resolve => {
            this.api.get(`/search?query=${query}`).subscribe(res => {
                resolve(res.data.results as SearchResult[]);
            });
        });
    }

    async openResult(result: SearchResult) {
        const nodeTypes = [
            'node', 'code', 'files', 'markdown',
        ];

        if ( result.type === 'page' ) {
            await this.router.navigate(['/editor', { id: result.id }]);
            await this.dismiss();
        } else if ( nodeTypes.includes(result.type) ) {
            await this.router.navigate(['/editor', { id: result.associated.id, node_id: result.id }]);
            await this.dismiss();
        } else if ( result.type === 'db' ) {
            await this.openDatabase(result.associated.id, result.id);
        } else if ( result.type.startsWith('file_box') ) {
            await this.openFileBox(result.associated.id, result.id, result.boxId);
        }
    }

    async openRelated(event, result: SearchResult) {
        event.preventDefault();
        event.stopPropagation();

        if ( result.associated ) {
            if ( result.associated.type === 'page' ) {
                await this.router.navigate(['/editor', { id: result.associated.id }]);
                await this.dismiss();
            }
        }
    }

    async openDatabase(pageId: string, nodeId: string) {
        const modal = await this.ionModalController.create({
            component: DatabasePageComponent,
            componentProps: {
                nodeId,
                pageId,
            },
            cssClass: 'modal-big',
        });

        const modalState = {
            modal : true,
            desc : 'Open Database'
        };

        history.pushState(modalState, null);

        await modal.present();
    }

    async openFileBox(pageId: string, nodeId: string, boxId?: string) {
        const modal = await this.ionModalController.create({
            component: FileBoxPageComponent,
            componentProps: {
                nodeId,
                pageId,
                boxId,
            },
            cssClass: 'modal-big',
        });

        const modalState = {
            modal : true,
            desc : 'Open File Box'
        };

        history.pushState(modalState, null);

        await modal.present();
    }
}
